@@ -29,6 +29,8 @@
 
 #include <KLocalizedString>
 #include <KUrl>
+#include <KRun>
+#include <KShell>
 #include <KPluginFactory>
 
 #include <QTemporaryFile>
@@ -53,12 +55,24 @@ FileViewGitPlugin::FileViewGitPlugin(QObject* parent, const QList<QVariant>& arg
 {
     Q_UNUSED(args);
 
+    m_revertAction = new QAction(this);
+    m_revertAction->setIcon(QIcon::fromTheme("document-revert"));
+    m_revertAction->setText(xi18nd("@action:inmenu", "<application>Git</application> Revert"));
+    connect(m_revertAction, SIGNAL(triggered()),
+            this, SLOT(revertFiles()));
+
     m_addAction = new QAction(this);
     m_addAction->setIcon(QIcon::fromTheme("list-add"));
     m_addAction->setText(xi18nd("@action:inmenu", "<application>Git</application> Add"));
     connect(m_addAction, SIGNAL(triggered()),
             this, SLOT(addFiles()));
 
+    m_showLocalChangesAction = new QAction(this);
+    m_showLocalChangesAction->setIcon(QIcon::fromTheme("view-split-left-right"));
+    m_showLocalChangesAction->setText(xi18nd("@item:inmenu", "Show Local <application>Git</application> Changes"));
+    connect(m_showLocalChangesAction, SIGNAL(triggered()),
+            this, SLOT(showLocalChanges()));
+
     m_removeAction = new QAction(this);
     m_removeAction->setIcon(QIcon::fromTheme("list-remove"));
     m_removeAction->setText(xi18nd("@action:inmenu", "<application>Git</application> Remove"));
@@ -281,6 +295,7 @@ QList<QAction*> FileViewGitPlugin::contextMenuFilesActions(const KFileItemList&
         //see which actions should be enabled
         int versionedCount = 0;
         int addableCount = 0;
+        int revertCount = 0;
         foreach(const KFileItem& item, items){
             const ItemVersion state = itemVersion(item);
             if (state != UnversionedVersion && state != RemovedVersion &&
@@ -291,19 +306,26 @@ QList<QAction*> FileViewGitPlugin::contextMenuFilesActions(const KFileItemList&
                 state == IgnoredVersion) {
                 ++addableCount;
             }
+            if (state == LocallyModifiedVersion || state == LocallyModifiedUnstagedVersion ||
+                state == ConflictingVersion) {
+                ++revertCount;
+            }
         }
 
         m_addAction->setEnabled(addableCount == items.count());
+        m_revertAction->setEnabled(revertCount == items.count());
         m_removeAction->setEnabled(versionedCount == items.count());
     }
     else{
         m_addAction->setEnabled(false);
+        m_revertAction->setEnabled(false);
         m_removeAction->setEnabled(false);
     }
 
     QList<QAction*> actions;
     actions.append(m_addAction);
     actions.append(m_removeAction);
+    actions.append(m_revertAction);
     return actions;
 }
 
@@ -317,19 +339,27 @@ QList<QAction*> FileViewGitPlugin::contextMenuDirectoryActions(const QString& di
     actions.append(m_checkoutAction);
 
     bool canCommit = false;
+    bool showChanges = false;
     QHash<QString, ItemVersion>::const_iterator it = m_versionInfoHash.constBegin();
     while (it != m_versionInfoHash.constEnd()) {
         const ItemVersion state = it.value();
         if (state == LocallyModifiedVersion || state == AddedVersion || state == RemovedVersion) {
             canCommit = true;
         }
+        if (state == LocallyModifiedUnstagedVersion || state == LocallyModifiedVersion) {
+            showChanges = true;
+        }
         if (state == ConflictingVersion) {
             canCommit = false;
+            showChanges = true;
             break;
         }
         ++it;
     }
 
+    m_showLocalChangesAction->setEnabled(!m_pendingOperation && showChanges);
+    actions.append(m_showLocalChangesAction);
+
     m_commitAction->setEnabled(!m_pendingOperation && canCommit);
     actions.append(m_commitAction);
 
@@ -360,7 +390,21 @@ void FileViewGitPlugin::removeFiles()
                    xi18nd("@info:status", "Removing files from <application>Git</application> repository..."),
                    xi18nd("@info:status", "Removing files from <application>Git</application> repository failed."),
                    xi18nd("@info:status", "Removed files from <application>Git</application> repository."));
+}
 
+void FileViewGitPlugin::revertFiles()
+{
+    execGitCommand(QLatin1String("checkout -- "), QStringList(),
+                   xi18nd("@info:status", "Reverting files from <application>Git</application> repository..."),
+                   xi18nd("@info:status", "Reverting files from <application>Git</application> repository failed."),
+                   xi18nd("@info:status", "Reverted files from <application>Git</application> repository."));
+}
+
+void FileViewGitPlugin::showLocalChanges()
+{
+    Q_ASSERT(!m_contextDir.isEmpty());
+
+    KRun::runCommand(QLatin1String("git difftool --dir-diff ."), nullptr, m_contextDir);
 }
 
 void FileViewGitPlugin::checkout()
